#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Dec 24 17:40:52 2019

@author: renyangying
"""
import fix_yahoo_finance as yf
import pandas as pd
pd.core.common.is_list_like=pd.api.types.is_list_like
import matplotlib.pyplot as plt
import matplotlib.mlab as mlab
import numpy as np
from scipy.stats import norm
import scipy.optimize as scp
import statsmodels.api as sm
stocks=pd.read_excel('stocks.xlsx')
stocks=stocks[0:30]
tickers=stocks.iloc[:,0]


#input the stocks data from excel

start_date='2010-01-01'
end_date='2017-01-01'
panel_data=pd.DataFrame()
for i in range(tickers.index[0],tickers.index[len(tickers)-1]+1):
    try:
        data=yf.download(tickers[i],start_date,end_date)
        data=data['Adj Close']
        data=pd.Series.to_frame(data)
        data.columns=[tickers[i]]
        panel_data=pd.concat([panel_data,data],axis=1)
    except ValueError:
        pass 
#download prices data from yahoo finance, if the column is empty pass it.
risk_free=yf.download('^TNX',start_date,end_date)
risk_free=(risk_free['Adj Close']/100)/365
risk_free=pd.Series.to_frame(risk_free)
risk_fr=risk_free.mean()
#calculate the risk free rate
df_price=panel_data
number=len(df_price.columns)
df_means=np.log(df_price/df_price.shift(1))
risk_free=risk_free.reindex(df_means.index)
rf=pd.Series(risk_free['Adj Close'],index=risk_free.index)
df_adjust_m=pd.DataFrame()
for i in range(tickers.index[0],tickers.index[len(tickers)-1]+1):
    dm=pd.Series(df_means[tickers[i]],index=df_means.index)
    dm=pd.Series.to_frame(dm-rf)
    dm.columns=[tickers[i]]
    df_adjust_m=pd.concat([df_adjust_m,dm],axis=1)
df_adjust_m.hist(bins=50,figsize=(9,6))
df_mean=df_adjust_m.mean()
#calculate the return of each stock
m=df_mean*(-1)
cov=df_adjust_m.cov()
for i in range(tickers.index[0],tickers.index[len(tickers)-1]+1):
    sm.qqplot(df_adjust_m[tickers[i]].dropna(),line='s')
    plt.grid(True)
#draw the qq plot 
port_return=[]
port_variance=[]
port_VaR=[]
port_ES=[]
p_std=[]
port_sharpe=[]
for p in range(1000):   
   weights=np.random.random(number)
   weights/=np.sum(weights)
   var=np.dot(weights,np.dot(cov,weights))
   std=np.sqrt(var)
   p_std.append(std)
   mean=np.dot(weights,m)
   sharpe_ratio=(mean*(-1)-risk_fr)/std
   VaR_90=mean+norm.ppf(0.90,0,1)*std
   VaR_95=mean+norm.ppf(0.99,0,1)*std
   VaR_99=mean+norm.ppf(0.99,0,1)*std
   ES_90=mean+std*norm.pdf(norm.ppf(0.90,0,1),0,1)/0.1
   ES_95=mean+std*norm.pdf(norm.ppf(0.95,0,1),0,1)/0.05
   ES_99=mean+std*norm.pdf(norm.ppf(0.99,0,1),0,1)/0.01
   port_return=np.append(port_return,mean*(-1))
   port_variance=np.append(port_variance,std)
   port_VaR=np.append(port_VaR,VaR_95)
   port_ES=np.append(port_ES,ES_95)
   port_return=np.array(port_return)
   port_variance=np.array(port_variance)
   port_VaR=np.array(port_VaR)
   port_es=np.array(port_ES)
   port_sharpe=np.append(port_sharpe,sharpe_ratio)
   port_sharpe=np.array(port_sharpe)
def optimal(weights):
    weights=np.array(weights)
    port_return=np.dot(weights,df_mean)
    port_variance=np.sqrt(np.dot(weights,np.dot(cov,weights)))
    port_VaR=port_return*(-1)+port_variance*norm.ppf(0.95,0,1)
    port_ES=port_return*(-1)+port_variance*norm.pdf(norm.ppf(0.95,0,1))/0.05
    sharpe_ratio=(port_return-risk_fr)/port_variance
    return np.array([port_return,port_variance,port_VaR,port_ES,sharpe_ratio]) 
#function of calculate parameters 
def min_variance(weights):
    return optimal(weights)[1]
def min_VaR(weights):
    return optimal(weights)[2]
def min_es(weights):
    return optimal(weights)[3]

cons = ({'type':'eq', 'fun':lambda x: np.sum(x)-1})
bnds = tuple((0,1) for x in range(number))
opts_variance=scp.minimize(min_variance, number*[1./number,],method = 'SLSQP', bounds = bnds, constraints = cons)
opts_VaR=scp.minimize(min_VaR, number*[1./number,],method = 'SLSQP', bounds = bnds, constraints = cons)
opts_es=scp.minimize(min_es, number*[1./number,],method = 'SLSQP', bounds = bnds, constraints = cons)
#Using SLSQP minimize to find the point with the minimum variance 计算最小点比例
opts_weights_es=opts_es['x'].round(3)
print("The weights of minimum risk is ")
print(opts_weights_es)
print(optimal(opts_es['x']).round(3))  
tar_return=port_return
tar_sharpe=port_sharpe
tar_variance=[]
tar_VaR=[]
tar_es=[] 
sample_weights=pd.DataFrame() 
for tar in tar_return:
    cons = ({'type':'eq','fun':lambda x:optimal(x)[0]-tar},{'type':'eq','fun':lambda x:np.sum(x)-1})
    res_es=scp.minimize(min_es, number*[1./number,],method = 'SLSQP', bounds = bnds, constraints = cons)
    tar_es=np.append(tar_es,res_es['fun'])
    sample_weights=pd.concat([sample_weights,pd.DataFrame(res_es['x'])],axis=1)
    tar_es=np.array(tar_es)
tar_sharpe_var=[]
#calculate the effecient frontier
for tar in tar_sharpe:
    cons = ({'type':'eq','fun':lambda x:optimal(x)[4]-tar},{'type':'eq','fun':lambda x:np.sum(x)-1})
    res_sharpe=scp.minimize(min_es, number*[1./number,],method = 'SLSQP', bounds = bnds, constraints = cons)
    tar_sharpe_var=np.append(tar_sharpe_var,res_sharpe['fun'])
    tar_sharpe_var=np.array(tar_sharpe_var)
sample_weight=opts_weights_es
print("The range of return is ")
print(min(port_return), max(port_return))
plt.figure(figsize=(8,4))
#plt.scatter(tar_sharpe_var,tar_sharpe,c='b',marker = '.',s=1)
plt.plot(optimal(opts_es['x'])[3],optimal(opts_es['x'])[0],'y*',markersize=20)
plt.scatter(tar_es,tar_return,c='y',marker = '.',s=1)
plt.grid(True)
plt.xlabel('Risk')
plt.ylabel('Return')

#draw the efficient frontier
